@@ -1,9 +1,10 @@
 package TAPx::Parser::Results::Comment;
 
 use strict;
-use warnings;
-use base 'TAPx::Parser::Results';
-use vars qw($VERSION);
+
+use vars qw($VERSION @ISA);
+use TAPx::Parser::Results;
+@ISA = 'TAPx::Parser::Results';
 
 =head1 NAME
 
@@ -11,11 +12,11 @@ TAPx::Parser::Results::Comment - Comment result token.
 
 =head1 VERSION
 
-Version 0.22
+Version 0.30
 
 =cut
 
-$VERSION = '0.22';
+$VERSION = '0.30';
 
 =head1 DESCRIPTION
 
